# -*- coding:utf-8 -*-

# responses.py
import os, sys
import re
import mimetypes
from django.http import HttpResponseRedirect, HttpResponse, HttpResponseNotFound, \
                        HttpResponseBadRequest, HttpResponseNotAllowed, HttpResponseForbidden, \
                        HttpResponseServerError, Http404, \
                        StreamingHttpResponse, JsonResponse

from krust.file_utils import *
from krux.random import random_str


def file_response(fname, save_name=None, content_type=None, remove_on_finish=False):
    del_tmp = False
    if re.match(r'^s3[c]?://.*', fname):
        try:
            tmpfile = '/tmp/tmp.%s%s' % (random_str(8), get_ext(fname))
            CP(fname, tmpfile)
            fname = tmpfile
            del_tmp = True
        except Exception as e:
           return HttpResponseNotFound()
    elif not os.path.exists(fname):
        return HttpResponseNotFound()

    if content_type is None:
        content_type, encoding = mimetypes.guess_type(fname)
    response = HttpResponse(open(fname, 'rb').read(), content_type)
    if save_name is None:
        response['Content-Disposition'] = 'attachment;'
    else:
        response['Content-Disposition'] = 'attachment; filename="%s"' % save_name
    if del_tmp and fname.startswith('/tmp/tmp.'):
        RM(fname)
    if remove_on_finish and os.path.isfile(fname):
        RM(fname)
    return response


def string_response(s, save_name=None, content_type=None):
    if save_name is None:
        save_name = 'tmpstr'

    if not content_type :
        if save_name:
            content_type, encoding = mimetypes.guess_type(save_name)
        else:
            content_type = 'application/octet-stream'
    else:
        if len(content_type) < 6:
            if not content_type.startswith('.'):
                content_type = '.' + content_type
            content_type, encoding = mimetypes.guess_type(content_type)

    response = HttpResponse(s, content_type)
    response['Content-Disposition'] = 'attachment; filename="%s"' % save_name

    return response


class ExceptionJsonResponse(JsonResponse):
    status_code = 400


class ErrorJsonResponse(JsonResponse):
    status_code = 500





__all__ = ['PageIndexPagination']

from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class PageIndexPagination(PageNumberPagination):
    page_size_query_param = 'page_size'

    def get_paginated_response(self, data):
        return Response({
            'current': self.page.number,
            'next': self.page.next_page_number() if self.page.has_next() else None,
            'previous': self.page.previous_page_number() if self.page.has_previous() else None,
            'num_pages': self.page.paginator.num_pages,
            'count': self.page.paginator.count,
            'results': data
        })



# -*- coding:utf-8 -*-

import six
from Crypto.Cipher import AES
import base64


__all__ = ['encrypt', 'decrypt']


def _pad_key(key):
    if len(key) > 32:
        return key[:32]
    elif len(key) > 24:
        return key.ljust(32)
    elif len(key) > 16:
        return key.ljust(24)
    else:
        return key.ljust(16)


def encrypt(msg, key=None, url_safe=False):
    if not key:
        return msg
    else:
        key = _pad_key(key)
        if not isinstance(key, bytes):
            key = key.encode('utf-8')

    msg = (msg + '$').encode('utf-8')
    padded_msg = msg.ljust((len(msg) // 16 + 1) * 16)
    cipher = AES.new(key, AES.MODE_ECB)
    encrypted = cipher.encrypt(padded_msg)
    if url_safe:
        encoded = base64.b64encode(encrypted, altchars=b'-_')
    else:
        encoded = base64.b64encode(encrypted)
    return encoded.decode('utf-8')


def decrypt(encoded, key=None, url_safe=False):
    if not key:
        if isinstance(encoded, bytes):
            return encoded.decode('utf-8')
        else:
            return encoded
    else:
        key = _pad_key(key)
        if not isinstance(key, bytes):
            key = key.encode('utf-8')

    cipher = AES.new(key, AES.MODE_ECB)
    if url_safe:
        decoded = base64.b64decode(encoded, altchars=b'-_')
    else:
        decoded = base64.b64decode(encoded)
    padded_msg = cipher.decrypt(decoded)
    msg = padded_msg.rstrip()
    msg = msg.decode('utf-8')
    msg = msg[:-1]
    return msg





# -*- coding:utf-8 -*-

import unittest
from datetime import datetime
from django.utils import timezone
from knob.dt import *


class TestDT(unittest.TestCase):
    def setUp(self):
        pass

    def test_local_time(self):
        for raw_time, expected_str in [
            ('20230101', '2023-01-01 00:00:00'),
            (timezone.make_aware(datetime(year=2023, month=1, day=1, hour=0), timezone=timezone.utc), '2023-01-01 08:00:00'),
        ]:
            t = local_time(raw_time)
            t_str = t.strftime('%Y-%m-%d %H:%M:%S')
            self.assertEqual(expected_str, t_str)
            self.assertEqual(t.tzinfo, timezone.get_current_timezone())
            self.assertTrue(timezone.is_aware(t))

        now = local_now()
        converted = local_time(now)
        self.assertTrue(timezone.is_aware(converted))


# -*- coding:utf-8 -*-
from django.utils.html import format_html


__all__ = ['rich_tag']


def rich_tag(text, color=None, bold=False, italic=False, hint=None, link=None, tag='span'):
    opts = {
        "text": text,
        "color": "color: {};".format(color) if color else "",
        "tag": tag if link is None else 'a',
        "bold": " font-weight: bold;" if bold else "",
        "italic": " font-style: italic;" if italic else "",
        "link": link,
    }

    if link is not None:
        link_tmpl = ' href="{link}"'
    else:
        link_tmpl = ''

    if hint:
        opts['hint'] = hint
        tmpl = u'<{tag} style="{color}{bold}{italic}" title="{hint}"' + link_tmpl + '>{text}</{tag}>'
    else:
        tmpl = u'<{tag} style="{color}{bold}{italic}"' + link_tmpl + '>{text}</{tag}>'

    return format_html(tmpl, **opts)



# -*- coding:utf-8 -*-

import os
import six
from django.conf import settings

__all__ = ['get_setting', 'set_setting', 'turn_on_debug', 'turn_off_debug', 'is_debug_on']

setting_pool = []
dynamic_setting = None
constance_config = None

_INITED = False

_DEBUG = 'DEBUG_KNOB_SETTING' in os.environ
_DEBUG_SETTINGS_DICT = {}


def turn_on_debug():
    global _DEBUG
    _DEBUG = True


def turn_off_debug():
    global _DEBUG
    _DEBUG = False
    _DEBUG_SETTINGS_DICT.clear()


def is_debug_on():
    return _DEBUG


def get_setting(key, default=None):
    global _INITED
    if not _INITED:
        _init_knob_setting()

    global setting_pool

    if is_debug_on() and key in _DEBUG_SETTINGS_DICT:
        return _DEBUG_SETTINGS_DICT[key]

    res = default
    for setting in setting_pool:
        if hasattr(setting, key):
            res = getattr(setting, key)
    return res


def set_setting(key, value):
    global _INITED
    if not _INITED:
        _init_knob_setting()

    global constance_config

    if is_debug_on():
        _DEBUG_SETTINGS_DICT[key] = value
    else:
        if dynamic_setting == 'Constance':
            setattr(constance_config, key, value)
        else:
            raise RuntimeError("No dynamic setting module installed.")


def _init_knob_setting():
    global dynamic_setting
    global constance_config
    global _INITED

    if 'constance' in settings.INSTALLED_APPS:
        try:
            from constance import config as constance_config_
            constance_config = constance_config_
            setting_pool.append(constance_config)
            dynamic_setting = 'Constance'
        except ImportError:
            constance_config = None

    setting_pool.append(settings)

    _INITED = True



import unittest
from knob.setting import *


class TestSetting(unittest.TestCase):
    def setUp(self):
        pass

    def test_preset_keys(self):
        self.assertEqual(get_setting('TEST_ENTRY_NUM'), 1)
        self.assertEqual(get_setting('TEST_ENTRY_STR'), 'abc')
        self.assertDictEqual(get_setting('TEST_ENTRY_DICT'), {"a": 1, "b": "abc"})

    def test_debug_mode(self):
        try:
            turn_off_debug()
            self.assertFalse(is_debug_on())

            turn_on_debug()
            self.assertTrue(is_debug_on())

            self.assertEqual(get_setting('TEST_ENTRY_NUM'), 1)

            set_setting("TEST_ENTRY_NUM", 2)
            self.assertEqual(get_setting('TEST_ENTRY_NUM'), 2)

            turn_off_debug()  # debugging settings should be cleared.

            self.assertEqual(get_setting('TEST_ENTRY_NUM'), 1)

            turn_on_debug()
            self.assertEqual(get_setting('TEST_ENTRY_NUM'), 1)
        finally:
            turn_off_debug()


if __name__ == '__main__':
    unittest.main()
